﻿using System;
using System.Web;
using System.Collections.Generic;
using System.Text;
using System.IO;
using XMailAdminProxy;

public class XMSettingsRow : IEquatable<XMSettingsRow>
{
	protected bool _isComment = false;
	protected string _key = null;
	protected string _value = null;

	public bool IsComment
	{
		get { return _isComment; }
		set { _isComment = value; }
	}

	public string Key
	{
		get { return _key; }
		set { _key = value; }
	}

	public string Value
	{
		get { return _value; }
		set { _value = value; }
	}

	public XMSettingsRow() { }
	public XMSettingsRow(bool isComment, string key, string value)
	{
		_isComment = isComment;
		_key = key;
		_value = value;
	}

	public static XMSettingsRow Parse(string row)
	{
		XMSettingsRow xmRow = null;
		if (!string.IsNullOrEmpty(row))
		{
			xmRow = new XMSettingsRow();
			string[] strings = row.Split(new char[] { '\t' });
			for (int i = 0; i < strings.Length; i++)
			{
				strings[i] = strings[i].Trim();
			}
			if (strings[0][0] == '#')
			{
				xmRow.IsComment = true;
				xmRow.Key = strings[0].Substring(1).Trim(new char[] { '"' });
			}
			else
			{
				xmRow.IsComment = false;
				xmRow.Key = strings[0].Trim(new char[] { '"' });
			}
			if (strings.Length > 1)
			{
				xmRow.Value = strings[1].Trim(new char[] { '"' });
			}
		}
		return xmRow;
	}

	public override string ToString()
	{
		StringBuilder sb = new StringBuilder();
		if (IsComment)
		{
			sb.Append("#");
		}
		if (!string.IsNullOrEmpty(Key))
		{
			sb.AppendFormat("\"{0}\"", Key);
			if (!string.IsNullOrEmpty(Value))
			{
				sb.AppendFormat("\t\"{0}\"", Value);
			}
		}
		return sb.ToString();
	}

	#region IEquatable<XMSettingsRow> Members

	public bool Equals(XMSettingsRow other)
	{
		if (string.Compare(Key, other.Key, StringComparison.InvariantCultureIgnoreCase) == 0)
		{
			return true;
		}
		return false;
	}

	#endregion
}

/// <summary>
/// Summary description for XMServerSettings
/// </summary>
public class XMServerSettings
{
	protected List<XMSettingsRow> _rows = new List<XMSettingsRow>();
    protected XMLayer _xm;

	public List<XMSettingsRow> Rows
	{
		get { return _rows; }
	}

    public XMLayer Xm
    {
        get { return _xm; }
        set { _xm = value; }
    }

    public XMServerSettings() 
    {
        _xm = Utils.CreateXMLayer();
    }

	public static XMServerSettings CreateXMServerSettings()
	{
		if ((HttpContext.Current != null))
		{
			if (HttpContext.Current.Application[AdminConstants.serverSettings] != null)
			{
				return (XMServerSettings)HttpContext.Current.Application[AdminConstants.serverSettings];
			}
		}
		XMServerSettings settings = new XMServerSettings {_xm = Utils.CreateXMLayer()};
	    settings.LoadSettings();
		if ((HttpContext.Current != null))
		{
			HttpContext.Current.Application.Add(AdminConstants.serverSettings, settings);
		}
		return settings;
	}

	public void LoadSettings()
	{
		try
		{
            _xm.Login();
            string raw = _xm.GetConfigFile(AdminConstants.serverTab);
            _xm.Logout();
            string line = string.Empty;

            using (StringReader sr = new StringReader(raw))
            {
                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    if (line == ".") break;
                    XMSettingsRow row = XMSettingsRow.Parse(line);
                    if (row != null)
                    {
                        _rows.Add(row);
                    }
                }
            }
		}
		catch (Exception)
		{
		}
	}

	public void SaveSettings()
	{
		try
		{
			using (StringWriter sw = new StringWriter())
			{
				for (int i = 0; i < _rows.Count; i++)
				{
					sw.WriteLine(_rows[i]);
				}
                string data = sw.ToString();
                _xm.Login();
                _xm.SetConfigFile(AdminConstants.serverTab, data);
                _xm.Logout();
            }
        }
		catch (Exception)
		{
			throw;
		}
	}

	public XMSettingsRow GetXMSettingsRow(string key)
	{
		XMSettingsRow result = null;
		int index = 0;
		XMSettingsRow item = new XMSettingsRow();
		item.Key = key;
		while ((index >= 0) && (index < _rows.Count))
		{
			index = _rows.IndexOf(item, index);
		    if (index <= 0) continue;
		    result = _rows[index];
		    if (!result.IsComment) break;
		    index++;
		}
		return result;
	}

	public void SetXMSettingsRow(XMSettingsRow row)
	{
		int index = _rows.IndexOf(row);
		if (index >= 0)
		{
			_rows[index].IsComment = row.IsComment;
			_rows[index].Key = row.Key;
			_rows[index].Value = row.Value;
		}
		else
		{
			_rows.Add(row);
		}
	}
}

public interface IXMSettingsCtrl
{
	void InitControl(XMServerSettings xmSettings);
	void SaveSettings(ref XMServerSettings xmSettings);
}
